/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package PointInTriangle;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Polygon;
import javax.swing.JPanel;

/**
 *
 * @author SUN2K
 */
public class DrawingPanel extends JPanel {
    
    /**
     * background color of applet
     */
    private final static Color backgroundColor = new Color(0, 120, 255);
    
    /**
     * triangle color if point is inside
     */
    private final static Color greenColor = new Color(0, 153, 51);
    
    /**
     * triangle color if point is outside
     */
    private final static Color redColor = new Color(255, 102, 102);
    
    /**
     * the three vertices of the triangle
     */
    private MovablePoint[] trianglePoints;
    
    /**
     * point to test includeness for
     */
    private MovablePoint testPoint;

    /**
     * available different algorithms
     */
    public enum ALGORITHM { CROSSPRODUCT_ALGO, BARYCENTRIC_ALGO };
    
    /**
     * currently selected algorithm to use
     */
    private ALGORITHM algorithm;
    
    /**
     * reference to parent applet for easier coding
     */
    private PointInTriangleApplet parentApplet;

    /**
     * Creates a new instance of DrawingPanel 
     * @param width
     * @param height 
     */
    public DrawingPanel(PointInTriangleApplet applet, int width, int height) {
        this.setLayout(null);
        this.setBounds(0, 0, width, height);
        this.setBackground(backgroundColor);
        this.algorithm = ALGORITHM.CROSSPRODUCT_ALGO;
        this.parentApplet = applet;
        
        /* initialize the beginning triangle */
        trianglePoints = new MovablePoint[3];
        trianglePoints[0] = new MovablePoint(this, width / 3, height / 10);
        this.add(trianglePoints[0]);
        trianglePoints[1] = new MovablePoint(this, width / 5, height / 10 * 8);
        this.add(trianglePoints[1]);
        trianglePoints[2] = new MovablePoint(this, width / 4 * 3, height / 10 * 6);
        this.add(trianglePoints[2]);
        
        /* add the test point */
        testPoint = new MovablePoint(this, width / 2, height / 2);
        testPoint.setBackground(Color.LIGHT_GRAY);
        this.add(testPoint);
    }
    
    /**
     * Helper function to fill triangle.
     * @param g Grpahics object
     * @param color fill color
     */
    private void fillTriangle(Graphics g, Color color)
    {
        Polygon poly = new Polygon();
        poly.addPoint(trianglePoints[0].getMidX(), trianglePoints[0].getMidY());
        poly.addPoint(trianglePoints[1].getMidX(), trianglePoints[1].getMidY());
        poly.addPoint(trianglePoints[2].getMidX(), trianglePoints[2].getMidY());
        Color oldColor = g.getColor();
        g.setColor(color);
        g.fillPolygon(poly);
        g.setColor(oldColor);
    }
    
    /**
     * Override for easy painting.
     * This is called every time a movable point changes its position.
     */
    @Override
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        
        
        
        /* perform actual check */
        boolean pInTriangle = checkPointInTriangle(new Point(trianglePoints[0].getMidX(), trianglePoints[0].getMidY()),
                                                   new Point(trianglePoints[1].getMidX(), trianglePoints[1].getMidY()),
                                                   new Point(trianglePoints[2].getMidX(), trianglePoints[2].getMidY()),
                                                   new Point(testPoint.getMidX(), testPoint.getMidY()));    
        
        if (pInTriangle)
        {
            fillTriangle(g, greenColor);
        }
        else
        {
            fillTriangle(g, redColor);
        }
        
        /* draw border of  triangle */
        g.drawLine(trianglePoints[0].getMidX(), trianglePoints[0].getMidY(), trianglePoints[1].getMidX(), trianglePoints[1].getMidY());
        g.drawLine(trianglePoints[1].getMidX(), trianglePoints[1].getMidY(), trianglePoints[2].getMidX(), trianglePoints[2].getMidY());
        g.drawLine(trianglePoints[2].getMidX(), trianglePoints[2].getMidY(), trianglePoints[0].getMidX(), trianglePoints[0].getMidY());

        for (int i = 0; i < trianglePoints.length; i++)
        {
            g.drawString("v" + i, trianglePoints[i].getX() +  trianglePoints[i].getWidth() + 5, trianglePoints[i].getY() + trianglePoints[i].getHeight());
        }
        
        printInfo();
    }

    /**
     * Check if given point is in triangle depending on active algorithm.
     * The vertices of the triangle do not need a special order.
     * @param pt1 first vertice of triangle
     * @param pt2 second vertice of triangle
     * @param pt3 third vertice of triangle
     * @param p point to test
     * @return true of point is inside triangle, false otherwise
     */
    private boolean checkPointInTriangle(Point pt1, Point pt2, Point pt3, Point p) {
        
        switch (algorithm)
        {
            case CROSSPRODUCT_ALGO:
                return checkPointInTriangleUsingCrossProduct(pt1, pt2, pt3, p);
                
            case BARYCENTRIC_ALGO:
                return checkPointInTriangleBarycentric(pt1, pt2, pt3, p);
                
            default: break;
        
        }
        
        return false;
    }
    
    /**
     * Check if point is in triangle using the three sided crossproduct algorithm.
     * @param pt1 first vertice of triangle
     * @param pt2 second vertice of triangle
     * @param pt3 third vertice of triangle
     * @param p point to test
     * @return true of point is inside triangle, false otherwise
     */
    private boolean checkPointInTriangleUsingCrossProduct(Point pt1, Point pt2, Point pt3, Point p) {
        int u1 = crossProduct( new Point(pt2.x - pt1.x, pt2.y - pt1.y),
                             new Point(p.x - pt1.x, p.y - pt1.y));
        int u2 = crossProduct( new Point(pt3.x - pt2.x, pt3.y - pt2.y),
                             new Point(p.x - pt2.x, p.y - pt2.y));
        int u3 = crossProduct( new Point(pt1.x - pt3.x, pt1.y - pt3.y),
                             new Point(p.x - pt3.x, p.y - pt3.y));
        
        parentApplet.getUserDefinedLabel1().setText("u1 = " + u1);
        parentApplet.getUserDefinedLabel2().setText("u2 = " + u2);
        parentApplet.getUserDefinedLabel3().setText("u3 = " + u3);
        
        if ( (u1 > 0 && u2 > 0 && u3 > 0) || (u1 < 0 && u2 < 0 && u3 < 0) )
        {
            /* point is inside the triangle */
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Calculates crossProduct of two 2D vectors / points.
     * @param p1 first point used as vector
     * @param p2 second point used as vector
     * @return crossProduct of vectors
     */
    private int crossProduct(Point p1, Point p2)
    {
        return (p1.x * p2.y - p1.y * p2.x);
    }
    
    /**
     * Check if point is in triangle using barycentric coordinates.
     * @param pt1 first vertice of triangle
     * @param pt2 second vertice of triangle
     * @param pt3 third vertice of triangle
     * @param p point to test
     * @return true of point is inside triangle, false otherwise
     */
    private boolean checkPointInTriangleBarycentric(Point pt1, Point pt2, Point pt3, Point p) 
    {
        Point v1 = new Point(pt2.x - pt1.x, pt2.y - pt1.y);
        Point v2 = new Point(pt3.x - pt1.x, pt3.y - pt1.y);
        
        Point q = new Point(p.x - pt1.x, p.y - pt1.y);
        
        float s = (float)crossProduct(q, v2) / crossProduct(v1, v2);
        float t = (float)crossProduct(v1, q) / crossProduct(v1, v2);
        
        parentApplet.getUserDefinedLabel1().setText("s = " + s);
        parentApplet.getUserDefinedLabel2().setText("t = " + t);
        parentApplet.getUserDefinedLabel3().setText("");
        
        if ( (s >= 0) && (t >= 0) && (s + t <= 1))
        {
            /* point is inside the triangle */
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Set new algorithm to use
     * @param algoMode new alogorithm
     */
    public void setAlgorithm(ALGORITHM algoMode)
    {
        switch (algoMode)
        {
            case CROSSPRODUCT_ALGO:
                algorithm = ALGORITHM.CROSSPRODUCT_ALGO;
                break;
                
            case BARYCENTRIC_ALGO:
                algorithm = ALGORITHM.BARYCENTRIC_ALGO;
                break;
                
            default: break;
        }
        
        repaint();
    }
    
    /**
     * Dirty helper function to print intermediate results
     */
    private void printInfo() {
        parentApplet.getTrianglePoint1XLabel().setText("x = " + trianglePoints[0].getMidX());
        parentApplet.getTrianglePoint1YLabel().setText("y = " + trianglePoints[0].getMidY());
        parentApplet.getTrianglePoint2XLabel().setText("x = " + trianglePoints[1].getMidX());
        parentApplet.getTrianglePoint2YLabel().setText("y = " + trianglePoints[1].getMidY());
        parentApplet.getTrianglePoint3XLabel().setText("x = " + trianglePoints[2].getMidX());
        parentApplet.getTrianglePoint3YLabel().setText("y = " + trianglePoints[2].getMidY());
        parentApplet.getTestPointXLabel().setText("y = " + testPoint.getMidX());
        parentApplet.getTestPointYLabel().setText("y = " + testPoint.getMidY());        
    }
    

}
